Imports System.Xml

Public Class MapRegion
    Public Pts() As Point
    Public Neighbors As Collection
    Public RegionColor As Color

    Public Number As Integer    ' Our number.

    Private NeighborsCopy As Collection

    ' See if rgn is a neighbor of ours.
    Public Sub CheckNeighbor(ByVal rgn As MapRegion)
        Dim i As Integer
        Dim x1 As Single
        Dim y1 As Single
        Dim x2 As Single
        Dim y2 As Single

        x1 = Pts(0).X
        y1 = Pts(0).Y
        For i = 1 To Pts.Length - 1
            x2 = Pts(i).X
            y2 = Pts(i).Y
            If rgn.HasLine(x1, y1, x2, y2) Then
                ' It's a neighbor.
                Neighbors.Add(rgn, rgn.Number.ToString("r#"))
                rgn.Neighbors.Add(Me, Number.ToString("r#"))
                Exit Sub
            End If
            x1 = x2
            y1 = y2
        Next i
    End Sub

    ' Identify this region with rgn. Copy this
    ' node's neighbors into rgn's neighbor list.
    Public Sub AssociateWith(ByVal rgn As MapRegion)
        Dim n1 As MapRegion
        Dim numtxt As String

        numtxt = rgn.Number.ToString("r#")
        For Each n1 In Neighbors
            If Not (n1 Is rgn) Then
                Try
                    ' Add the neighbor to rgn's list.
                    rgn.Neighbors.Add(n1, n1.Number.ToString("r#"))

                    ' It worked. Add rgn to n1's list.
                        n1.Neighbors.Add(rgn, numtxt)
                Catch ex As Exception
                End Try
            End If
        Next n1
    End Sub

    ' Remove the node from the graph.
    Public Sub RemoveFromGraph()
        Dim rgn As MapRegion
        Dim numtxt As String

        ' Remove links from neighbors to here.
        numtxt = Number.ToString("r#")
        For Each rgn In Neighbors
            rgn.Neighbors.Remove(numtxt)
        Next rgn

        ' Remove links to neighbors.
        Neighbors = Nothing
    End Sub

    ' Save the neighbor list for later.
    Public Sub SaveNeighbors()
        NeighborsCopy = New Collection
        For Each rgn As MapRegion In Neighbors
            NeighborsCopy.Add(rgn)
        Next rgn
    End Sub

    ' Restore the saved neighbor list.
    Public Sub RestoreNeighbors()
        Neighbors = New Collection
        For Each rgn As MapRegion In NeighborsCopy
            Neighbors.Add(rgn, rgn.Number.ToString("r#"))
        Next rgn
        NeighborsCopy = Nothing
    End Sub

    ' Return true if this region includes the line.
    Public Function HasLine(ByVal a1 As Single, ByVal b1 As Single, ByVal a2 As Single, ByVal b2 As Single) As Boolean
        Dim i As Integer
        Dim x1 As Single
        Dim y1 As Single
        Dim x2 As Single
        Dim y2 As Single

        ' Assume we will find it.
        HasLine = True

        x1 = Pts(0).X
        y1 = Pts(0).Y
        For i = 1 To Pts.Length - 1
            x2 = Pts(i).X
            y2 = Pts(i).Y
            If x1 = a1 And y1 = b1 And _
               x2 = a2 And y2 = b2 _
                    Then Exit Function
            If x2 = a1 And y2 = b1 And _
               x1 = a2 And y1 = b2 _
                    Then Exit Function
            x1 = x2
            y1 = y2
        Next i

        ' We didn't find the line.
        HasLine = False
    End Function

    ' Draw the region.
    Public Sub DrawRegion(ByVal gr As Graphics)
        gr.FillPolygon(New SolidBrush(RegionColor), Pts)
        gr.DrawPolygon(Pens.Black, Pts)
    End Sub

    ' Find two mutually non-adjacent neighbors with
    ' degree <= 7.
    Public Function FindNonAdjacents(ByRef n1 As MapRegion, ByRef n2 As MapRegion) As Boolean
        Dim n3 As MapRegion
        Dim non_adjacent As Boolean

        FindNonAdjacents = True
        For Each n1 In Neighbors
            ' See if n1 has degree <= 7.
            If n1.Neighbors.Count <= 7 Then
                For Each n2 In Neighbors
                    If Not (n1 Is n2) Then
                        ' See if n2 has degree <= 7.
                        If n2.Neighbors.Count <= 7 Then
                            ' See if n1 and n2 are
                            ' non-adjacent.
                            non_adjacent = True
                            For Each n3 In n1.Neighbors
                                If n3 Is n2 Then
                                    non_adjacent = False
                                    Exit For
                                End If
                            Next n3
                            If non_adjacent Then Exit Function
                        End If ' End if Degree(n2) <= 7.
                    End If ' End if n1 <> n2.
                Next n2
            End If ' End if Degree(n1) <= 7.
        Next n1
        FindNonAdjacents = False
    End Function

    ' Load the region's data from a file.
    Public Sub LoadData(ByVal xml_node As XmlElement)
        ' See how many points there are.
        Dim num_points As Integer = 0

        For Each point_node As XmlElement In xml_node.ChildNodes
            num_points += 1
            ReDim Preserve Pts(num_points - 1)
            With Pts(num_points - 1)
                .X = point_node.GetAttribute("X")
                .Y = point_node.GetAttribute("Y")
            End With
        Next point_node
    End Sub
End Class
